Index

Struct Index 

pub struct Index { /* private fields */ }
An index built for fast convertion between byte offsets and line-column locations.

NOTE: the last element of line_offsets should be considered as a fake offset (sentinel):

self.line_offsets: [line0, line1, ..., EOF]

By the word ‘fake’, we means that if some offset >= the last offset, it should be seen as exceding the index since the last element only marks ending.

Invariants

  • index is never empty.
  • index[0] == 0.
  • index is monotonically increasing.
  • index is append-only (no removals or mutations).
  • A sentinel EOF offset is always present as the last element.

Therefore:

  • Valid logical line indices are 0..self.count(), where count() = index.len() - 1.
  • For any valid line i, the byte range is: [index[i], index[i + 1]).

Implementations

Source

impl Index

pub fn new() -> Self

An index with the first line starting at offset 0, which is the most common usage.

The zero is safe here since it just means an ending, which also means empty.

pub fn count(&self) -> usize

length of index The API has guaranteed that self.line_offsets.len() > 0

pub fn end(&self) -> Option<Offset>

ending offset of the source

pub fn into_offsets(self) -> Vec<Offset>

into vector of offsets

Source

impl Index

pub fn query(&self) -> Query<'_>

Get the query and freeze index when querying

pub fn get_line_offset_mut(&mut self, line_no: usize) -> Option<&mut Offset>

pub fn add_next_line(&mut self, offset: Offset)

Add next line offset to index

pub fn clear(&mut self)

Reset the index